import _extends from "@babel/runtime/helpers/esm/extends";
import _toConsumableArray from "@babel/runtime/helpers/esm/toConsumableArray";
import _slicedToArray from "@babel/runtime/helpers/esm/slicedToArray";

/* eslint-disable @typescript-eslint/no-shadow */

/* eslint-disable no-plusplus */
import React, { forwardRef, useContext, useEffect, useImperativeHandle, useMemo, useRef, useState } from 'react';
import classnames from 'classnames';
import { Popup } from '../popup';
import Button from '../button';
import Toast from '../toast/index';
import CalendarHeader from './CalendarHeader';
import CalendarMonth from './CalendarMonth';
import { useUpdateEffect } from '../hooks';
import { getRect } from '../hooks/get-rect';
import useRefs from '../hooks/use-refs';
import { calcDateNum, cloneDate, cloneDates, compareDay, compareMonth, getDayByOffset, getNextDay, getPrevDay, getToday, t } from './utils';
import { getScrollTop, pick } from '../utils';
import { raf } from '../utils/raf';
import ConfigProviderContext from '../config-provider/ConfigProviderContext';
var Calendar = /*#__PURE__*/forwardRef(function (props, ref) {
  var _useContext = useContext(ConfigProviderContext),
      prefixCls = _useContext.prefixCls,
      createNamespace = _useContext.createNamespace;

  var _createNamespace = createNamespace('calendar', prefixCls),
      _createNamespace2 = _slicedToArray(_createNamespace, 1),
      bem = _createNamespace2[0];

  var type = props.type,
      title = props.title,
      showTitle = props.showTitle,
      showSubtitle = props.showSubtitle,
      firstDayOfWeek = props.firstDayOfWeek,
      minDate = props.minDate,
      maxDate = props.maxDate,
      safeAreaInsetBottom = props.safeAreaInsetBottom,
      footer = props.footer,
      showConfirm = props.showConfirm,
      className = props.className,
      style = props.style;

  var limitDateRange = function limitDateRange(date) {
    var minDate = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : props.minDate;
    var maxDate = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : props.maxDate;

    if (compareDay(date, minDate) === -1) {
      return minDate;
    }

    if (compareDay(date, maxDate) === 1) {
      return maxDate;
    }

    return date;
  };

  var getInitialDate = function getInitialDate() {
    var defaultDate = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : props.defaultDate;

    if (defaultDate === null) {
      return defaultDate;
    }

    var now = getToday();

    if (type === 'range') {
      if (!Array.isArray(defaultDate)) {
        defaultDate = [];
      }

      var start = limitDateRange(defaultDate[0] || now, minDate, getPrevDay(maxDate));
      var end = limitDateRange(defaultDate[1] || now, getNextDay(minDate));
      return [start, end];
    }

    if (type === 'multiple') {
      if (Array.isArray(defaultDate)) {
        return defaultDate.map(function (date) {
          return limitDateRange(date);
        });
      }

      return [limitDateRange(now)];
    }

    if (!defaultDate || Array.isArray(defaultDate)) {
      defaultDate = now;
    }

    return limitDateRange(defaultDate);
  };

  var bodyRef = useRef(null);
  var bodyHeightRef = useRef(0);

  var _useState = useState(''),
      _useState2 = _slicedToArray(_useState, 2),
      subtitle = _useState2[0],
      setSubtitle = _useState2[1];

  var _useState3 = useState(getInitialDate()),
      _useState4 = _slicedToArray(_useState3, 2),
      currentDate = _useState4[0],
      setCurrentDate = _useState4[1];

  var _useRefs = useRefs(),
      _useRefs2 = _slicedToArray(_useRefs, 2),
      monthRefs = _useRefs2[0],
      setMonthRefs = _useRefs2[1];

  var dayOffset = useMemo(function () {
    return firstDayOfWeek ? +firstDayOfWeek % 7 : 0;
  }, [firstDayOfWeek]);
  var monthsList = useMemo(function () {
    var months = [];
    var cursor = new Date(minDate);
    cursor.setDate(1);

    do {
      months.push(new Date(cursor));
      cursor.setMonth(cursor.getMonth() + 1);
    } while (compareMonth(cursor, maxDate) !== 1);

    return months;
  }, [minDate, maxDate]);
  var buttonDisabled = useMemo(function () {
    if (currentDate) {
      if (props.type === 'range') {
        return !currentDate[0] || !currentDate[1];
      }

      if (props.type === 'multiple') {
        return !currentDate.length;
      }
    }

    return !currentDate;
  }, [props.type, currentDate]);

  var onScroll = function onScroll() {
    if (!bodyRef.current) return;
    var top = getScrollTop(bodyRef.current);
    var bottom = top + bodyHeightRef.current;
    var heights = monthsList.map(function (_, index) {
      return monthRefs[index].getHeight();
    });
    var heightSum = heights.reduce(function (a, b) {
      return a + b;
    }, 0); // iOS scroll bounce may exceed the range

    if (bottom > heightSum && top > 0) {
      return;
    }

    var height = 0;
    var currentMonth;
    var visibleRange = [-1, -1];

    for (var i = 0; i < monthsList.length; i++) {
      var month = monthRefs[i];
      var visible = height <= bottom && height + heights[i] >= top;

      if (visible) {
        visibleRange[1] = i;

        if (!currentMonth) {
          currentMonth = month;
          visibleRange[0] = i;
        }

        if (!monthRefs[i].showed) {
          var _props$onMonthShow;

          monthRefs[i].showed = true;
          (_props$onMonthShow = props.onMonthShow) === null || _props$onMonthShow === void 0 ? void 0 : _props$onMonthShow.call(props, {
            date: month.date,
            title: month.getTitle()
          });
        }
      }

      height += heights[i];
    }

    monthsList.forEach(function (_, index) {
      var visible = index >= visibleRange[0] - 1 && index <= visibleRange[1] + 1;
      monthRefs[index].setVisible(visible);
    });
    /* istanbul ignore else */

    if (currentMonth) {
      setSubtitle(currentMonth.getTitle());
    }
  };

  var scrollToDate = function scrollToDate(targetDate) {
    raf(function () {
      monthsList.some(function (month, index) {
        if (compareMonth(month, targetDate) === 0) {
          if (bodyRef.current) {
            monthRefs[index].scrollToDate(bodyRef.current, targetDate);
          }

          return true;
        }

        return false;
      });
      onScroll();
    });
  };

  var scrollToCurrentDate = function scrollToCurrentDate(date) {
    if (props.poppable && !props.show) {
      return;
    }

    var currentDate = date || getInitialDate();

    if (currentDate) {
      var targetDate = props.type === 'single' ? currentDate : currentDate[0];
      scrollToDate(targetDate);
    } else {
      raf(onScroll);
    }
  };

  var init = function init() {
    if (props.poppable && !props.show) {
      return;
    }

    raf(function () {
      bodyHeightRef.current = Math.floor(getRect(bodyRef.current).height);
      scrollToCurrentDate();
    });
  };

  var reset = function reset() {
    var date = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : getInitialDate();
    setCurrentDate(date);
    scrollToCurrentDate(date);
  };

  var onConfirm = function onConfirm(date) {
    var _props$onConfirm;

    (_props$onConfirm = props.onConfirm) === null || _props$onConfirm === void 0 ? void 0 : _props$onConfirm.call(props, cloneDates(date || currentDate));
  };

  var checkRange = function checkRange(date) {
    var maxRange = props.maxRange,
        rangePrompt = props.rangePrompt,
        showRangePrompt = props.showRangePrompt;

    if (maxRange && calcDateNum(date) > maxRange) {
      var _props$onOverRange;

      if (showRangePrompt) {
        Toast(rangePrompt || t('rangePrompt', maxRange));
      }

      (_props$onOverRange = props.onOverRange) === null || _props$onOverRange === void 0 ? void 0 : _props$onOverRange.call(props);
      return false;
    }

    return true;
  };

  var select = function select(date, complete) {
    var toSetCurrentDate = function toSetCurrentDate(date) {
      var _props$onSelect;

      setCurrentDate(date);
      (_props$onSelect = props.onSelect) === null || _props$onSelect === void 0 ? void 0 : _props$onSelect.call(props, cloneDates(date));
    };

    if (complete && props.type === 'range') {
      var valid = checkRange(date);

      if (!valid) {
        // auto selected to max range
        toSetCurrentDate([date[0], getDayByOffset(date[0], +props.maxRange - 1)]);
        return;
      }
    }

    toSetCurrentDate(date);

    if (complete && !props.showConfirm) {
      onConfirm(date);
    }
  };

  var onClickDay = function onClickDay(item) {
    if (props.readonly || !item.date) {
      return;
    }

    var date = item.date;
    var type = props.type;

    if (type === 'range') {
      if (!currentDate) {
        select([date]);
        return;
      }

      var _ref = currentDate,
          _ref2 = _slicedToArray(_ref, 2),
          startDay = _ref2[0],
          endDay = _ref2[1];

      if (startDay && !endDay) {
        var compareToStart = compareDay(date, startDay);

        if (compareToStart === 1) {
          select([startDay, date], true);
        } else if (compareToStart === -1) {
          select([date]);
        } else if (props.allowSameDay) {
          select([date, date], true);
        }
      } else {
        select([date]);
      }
    } else if (type === 'multiple') {
      if (!currentDate) {
        select([date]);
        return;
      }

      var dates = currentDate;
      var selectedIndex = dates.findIndex(function (dateItem) {
        return compareDay(dateItem, date) === 0;
      });

      if (selectedIndex !== -1) {
        var _props$onUnselect;

        var _dates$splice = dates.splice(selectedIndex, 1),
            _dates$splice2 = _slicedToArray(_dates$splice, 1),
            unselectedDate = _dates$splice2[0];

        (_props$onUnselect = props.onUnselect) === null || _props$onUnselect === void 0 ? void 0 : _props$onUnselect.call(props, cloneDate(unselectedDate));
        setCurrentDate(_toConsumableArray(dates));
      } else if (props.maxRange && dates.length >= props.maxRange) {
        Toast(props.rangePrompt || t('rangePrompt', props.maxRange));
      } else {
        select([].concat(_toConsumableArray(dates), [date]));
      }
    } else {
      select(date, true);
    }
  };

  var renderMonth = function renderMonth(date, index) {
    var showMonthTitle = index !== 0 || !props.showSubtitle;
    return /*#__PURE__*/React.createElement(CalendarMonth, _extends({
      key: index,
      ref: setMonthRefs(index),
      date: date,
      currentDate: currentDate,
      showMonthTitle: showMonthTitle,
      firstDayOfWeek: dayOffset
    }, pick(props, ['type', 'color', 'minDate', 'maxDate', 'showMark', 'formatter', 'rowHeight', 'lazyRender', 'showSubtitle', 'allowSameDay']), {
      onClick: onClickDay
    }));
  };

  var renderFooter = function renderFooter() {
    var disabled = buttonDisabled;
    var text = disabled ? props.confirmDisabledText : props.confirmText;
    return /*#__PURE__*/React.createElement("div", {
      className: classnames(bem('footer'), {
        'rc-safe-area-bottom': safeAreaInsetBottom
      })
    }, footer || (showConfirm ? /*#__PURE__*/React.createElement(Button, {
      round: true,
      block: true,
      type: "danger",
      color: props.color,
      className: classnames(bem('confirm')),
      disabled: buttonDisabled,
      nativeType: "button",
      onClick: function onClick() {
        return onConfirm();
      }
    }, text || '确认') : null));
  };

  var renderCalendar = function renderCalendar() {
    return /*#__PURE__*/React.createElement("div", {
      className: classnames(bem(), className),
      style: style
    }, /*#__PURE__*/React.createElement(CalendarHeader, {
      title: title,
      subtitle: props.subtitle || subtitle,
      showTitle: showTitle,
      showSubtitle: showSubtitle,
      firstDayOfWeek: firstDayOfWeek,
      onClickSubtitle: function onClickSubtitle(event) {
        var _props$onClickSubtitl;

        (_props$onClickSubtitl = props.onClickSubtitle) === null || _props$onClickSubtitl === void 0 ? void 0 : _props$onClickSubtitl.call(props, event);
      }
    }), /*#__PURE__*/React.createElement("div", {
      ref: bodyRef,
      className: classnames(bem('body')),
      onScroll: onScroll
    }, monthsList.map(renderMonth)), renderFooter());
  };

  useEffect(function () {
    init();
  }, [props.show]);
  useUpdateEffect(function () {
    reset(getInitialDate());
  }, [props.type, props.minDate, props.maxDate]);
  useUpdateEffect(function () {
    setCurrentDate(props.defaultDate);
    scrollToCurrentDate();
  }, [props.defaultDate]);
  useImperativeHandle(ref, function () {
    return {
      reset: reset,
      scrollToDate: scrollToDate
    };
  });
  return props.poppable ? /*#__PURE__*/React.createElement(Popup, {
    visible: props.show,
    className: classnames(bem('popup')),
    round: props.round,
    position: props.position,
    closeable: props.showTitle || props.showSubtitle,
    teleport: props.teleport,
    closeOnClickOverlay: props.closeOnClickOverlay,
    closeOnPopstate: props.closeOnPopstate,
    onClose: function onClose() {
      var _props$onClose;

      return (_props$onClose = props.onClose) === null || _props$onClose === void 0 ? void 0 : _props$onClose.call(props);
    }
  }, renderCalendar()) : /*#__PURE__*/React.createElement(React.Fragment, null, renderCalendar());
});
Calendar.defaultProps = {
  type: 'single',
  title: '日期选择',
  color: '#ee0a24',
  rowHeight: 64,
  poppable: true,
  lazyRender: true,
  showMark: true,
  showTitle: true,
  showSubtitle: true,
  showConfirm: true,
  showRangePrompt: true,
  readonly: false,
  firstDayOfWeek: 0,
  footer: null,
  minDate: getToday(),
  maxDate: function () {
    var now = getToday();
    return new Date(now.getFullYear(), now.getMonth() + 6, now.getDate());
  }(),
  show: false,
  position: 'bottom',
  round: true,
  closeOnPopstate: true,
  closeOnClickOverlay: true,
  safeAreaInsetBottom: true,
  maxRange: null,
  allowSameDay: false
};
export default Calendar;