import _slicedToArray from "@babel/runtime/helpers/esm/slicedToArray";
import React, { useContext, useMemo } from 'react';
import classNames from 'classnames';
import ConfigProviderContext from '../config-provider/ConfigProviderContext';

var CalendarDay = function CalendarDay(props) {
  var _useContext = useContext(ConfigProviderContext),
      prefixCls = _useContext.prefixCls,
      createNamespace = _useContext.createNamespace;

  var _createNamespace = createNamespace('calendar', prefixCls),
      _createNamespace2 = _slicedToArray(_createNamespace, 1),
      bem = _createNamespace2[0];

  var item = props.item,
      index = props.index,
      color = props.color,
      offset = props.offset,
      rowHeight = props.rowHeight;
  var calcStyle = useMemo(function () {
    var style = {
      height: rowHeight
    };

    if (item.type === 'placeholder') {
      style.width = '100%';
      return style;
    }

    if (index === 0) {
      style.marginLeft = "".concat(100 * offset / 7, "%");
    }

    if (color) {
      switch (item.type) {
        case 'end':
        case 'start':
        case 'start-end':
        case 'multiple-middle':
        case 'multiple-selected':
          style.background = color;
          break;

        case 'middle':
          style.color = color;
          break;

        default:
          break;
      }
    }

    return style;
  }, [item, index, color, offset, rowHeight]);

  var onClick = function onClick() {
    if (props.item.type !== 'disabled') {
      var _props$onClick;

      (_props$onClick = props.onClick) === null || _props$onClick === void 0 ? void 0 : _props$onClick.call(props, item);
    }
  };

  var renderTopInfo = function renderTopInfo() {
    var topInfo = props.item.topInfo;
    return topInfo || props.topInfoRender ? /*#__PURE__*/React.createElement("div", {
      key: String(index),
      className: classNames(bem('top-info'))
    }, props.topInfoRender ? props.topInfoRender(props.item) : topInfo) : null;
  };

  var renderBottomInfo = function renderBottomInfo() {
    var bottomInfo = props.item.bottomInfo;
    return bottomInfo || props.bottomInfoRender ? /*#__PURE__*/React.createElement("div", {
      key: String(index),
      className: classNames(bem('bottom-info'))
    }, props.bottomInfoRender ? props.bottomInfoRender(props.item) : bottomInfo) : null;
  };

  var renderContent = function renderContent() {
    var type = item.type,
        text = item.text;
    var Nodes = [renderTopInfo(), text, renderBottomInfo()];

    if (type === 'selected') {
      return /*#__PURE__*/React.createElement("div", {
        className: classNames(bem('selected-day')),
        style: {
          width: rowHeight,
          height: rowHeight,
          background: color
        }
      }, Nodes);
    }

    return Nodes;
  };

  var _props$item = props.item,
      type = _props$item.type,
      className = _props$item.className;
  return type === 'placeholder' ? /*#__PURE__*/React.createElement("div", {
    key: String(index),
    className: classNames(bem('day')),
    style: calcStyle
  }) : /*#__PURE__*/React.createElement("div", {
    role: "gridcell",
    style: calcStyle,
    className: classNames(bem('day', type), className),
    tabIndex: type === 'disabled' ? undefined : -1,
    onClick: onClick
  }, renderContent());
};

CalendarDay.defaultProps = {
  offset: 0
};
export default CalendarDay;