import _slicedToArray from "@babel/runtime/helpers/esm/slicedToArray";
import React, { forwardRef, useRef, useState, useMemo, useEffect, useImperativeHandle } from 'react';
import { Picker, useUpdateEffect } from '..';
import { padZero, range } from '../utils';
import { times } from './utils';
var TimePicker = /*#__PURE__*/forwardRef(function (props, ref) {
  var formatValue = function formatValue(value) {
    var minHour = props.minHour,
        maxHour = props.maxHour,
        minMinute = props.minMinute,
        maxMinute = props.maxMinute;

    if (!value) {
      value = "".concat(padZero(minHour), ":").concat(padZero(minHour));
    }

    var _value$split = value.split(':'),
        _value$split2 = _slicedToArray(_value$split, 2),
        hour = _value$split2[0],
        minute = _value$split2[1];

    hour = padZero(range(+hour, +minHour, +maxHour));
    minute = padZero(range(+minute, +minMinute, +maxMinute));
    return "".concat(hour, ":").concat(minute);
  };

  var picker = useRef(null);

  var _useState = useState(formatValue(props.value)),
      _useState2 = _slicedToArray(_useState, 2),
      currentDate = _useState2[0],
      setCurrentDate = _useState2[1];

  var currentDateRef = useRef(null);
  currentDateRef.current = currentDate;
  var ranges = useMemo(function () {
    return [{
      type: 'hour',
      range: [+props.minHour, +props.maxHour]
    }, {
      type: 'minute',
      range: [+props.minMinute, +props.maxMinute]
    }];
  }, [props.minHour, props.maxHour, props.minMinute, props.maxMinute]);
  var originColumns = useMemo(function () {
    return ranges.map(function (_ref) {
      var type = _ref.type,
          rangeArr = _ref.range;
      var values = times(rangeArr[1] - rangeArr[0] + 1, function (index) {
        return padZero(rangeArr[0] + index);
      });

      if (props.filter) {
        values = props.filter(type, values);
      }

      return {
        type: type,
        values: values
      };
    });
  }, [ranges, props.filter]);
  var columns = useMemo(function () {
    return originColumns.map(function (column) {
      return {
        values: column.values.map(function (value) {
          return props.formatter(column.type, value);
        })
      };
    });
  }, [originColumns]);

  var updateColumnValue = function updateColumnValue(date) {
    var pair = date ? date.split(':') : currentDate.split(':');
    var values = [props.formatter('hour', pair[0]), props.formatter('minute', pair[1])];
    setTimeout(function () {
      var _picker$current;

      (_picker$current = picker.current) === null || _picker$current === void 0 ? void 0 : _picker$current.setValues(values);
    }, 0);
  };

  var updateInnerValue = function updateInnerValue() {
    var _picker$current$getIn = picker.current.getIndexes(),
        _picker$current$getIn2 = _slicedToArray(_picker$current$getIn, 2),
        hourIndex = _picker$current$getIn2[0],
        minuteIndex = _picker$current$getIn2[1];

    var _originColumns = _slicedToArray(originColumns, 2),
        hourColumn = _originColumns[0],
        minuteColumn = _originColumns[1];

    var hour = hourColumn.values[hourIndex] || hourColumn.values[0];
    var minute = minuteColumn.values[minuteIndex] || minuteColumn.values[0];
    setCurrentDate(formatValue("".concat(hour, ":").concat(minute)));
    updateColumnValue(formatValue("".concat(hour, ":").concat(minute)));
  };

  var onConfirm = function onConfirm() {
    var _props$onConfirm;

    return (_props$onConfirm = props.onConfirm) === null || _props$onConfirm === void 0 ? void 0 : _props$onConfirm.call(props, currentDate);
  };

  var onCancel = function onCancel() {
    return props.onCancel();
  };

  var onChange = function onChange(value) {
    updateInnerValue();
    setTimeout(function () {
      var _props$onChange;

      (_props$onChange = props.onChange) === null || _props$onChange === void 0 ? void 0 : _props$onChange.call(props, currentDateRef);
    }, 0);
  };

  useEffect(function () {
    updateColumnValue();
  }, [columns]);
  useUpdateEffect(function () {
    updateInnerValue();
  }, [props.filter, props.maxHour, props.minMinute, props.maxMinute]);
  useUpdateEffect(function () {
    setTimeout(updateInnerValue, 0);
  }, [props.minHour]);
  useUpdateEffect(function () {
    var value = formatValue(props.value);

    if (value !== currentDate) {
      setCurrentDate(value);
      updateColumnValue(value);
    }
  }, [props.value]);
  useImperativeHandle(ref, function () {
    return {
      getPicker: function getPicker() {
        return picker.current;
      }
    };
  });
  return /*#__PURE__*/React.createElement(Picker, {
    ref: picker,
    columns: columns,
    readonly: props.readonly,
    onChange: onChange,
    onCancel: onCancel,
    onConfirm: onConfirm
  });
});
TimePicker.defaultProps = {
  minHour: 0,
  maxHour: 23,
  minMinute: 0,
  maxMinute: 59,
  formatter: function formatter(type, value) {
    return value;
  }
};
export default TimePicker;