import _defineProperty from "@babel/runtime/helpers/esm/defineProperty";
import _slicedToArray from "@babel/runtime/helpers/esm/slicedToArray";

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }

import React, { useEffect, useContext, useRef, useState, useMemo, createContext, forwardRef, useImperativeHandle } from 'react';
import classnames from 'classnames';
import ConfigProviderContext from '../config-provider/ConfigProviderContext';
import { HAPTICS_FEEDBACK } from '../utils/constant';
import { isDef } from '../utils';
import useScrollParent from '../hooks/use-scroll-parent';
import useClickAway from '../hooks/use-click-away';
import useEventListener from '../hooks/use-event-listener';
import useRefs from '../hooks/use-refs';
import getRect from '../hooks/get-rect';
export var DropdownMenuContext = /*#__PURE__*/createContext({});
var DropdownMenu = /*#__PURE__*/forwardRef(function (props, ref) {
  var _useContext = useContext(ConfigProviderContext),
      prefixCls = _useContext.prefixCls,
      createNamespace = _useContext.createNamespace;

  var _createNamespace = createNamespace('dropdown-menu', prefixCls),
      _createNamespace2 = _slicedToArray(_createNamespace, 1),
      bem = _createNamespace2[0];

  var _useState = useState(props.shouldRenderTitle),
      _useState2 = _slicedToArray(_useState, 2),
      titleFlag = _useState2[0],
      setTitleFlag = _useState2[1];

  var _useState3 = useState(null),
      _useState4 = _slicedToArray(_useState3, 2),
      showPopupIndex = _useState4[0],
      setShowPopupIndex = _useState4[1];

  var showPopupIndexRef = useRef(null);
  var rect = useRef({
    bottom: 0,
    top: 0
  });
  var root = useRef();
  var barRef = useRef();
  var offsetRef = useRef(0);

  var _useRefs = useRefs(),
      _useRefs2 = _slicedToArray(_useRefs, 2),
      childrenRefs = _useRefs2[0],
      setChildrenRefs = _useRefs2[1];

  var scrollParent = useScrollParent(root);
  var opened = useMemo(function () {
    return showPopupIndex !== null;
  }, [showPopupIndex]);

  var barStyle = function barStyle() {
    if (opened && isDef(props.zIndex)) {
      return {
        zIndex: +props.zIndex + 1
      };
    }

    return {};
  };

  var updateShowPopupIndex = function updateShowPopupIndex(current) {
    showPopupIndexRef.current = current;
    setShowPopupIndex(current);
  };

  var updateOffset = function updateOffset() {
    if (barRef.current) {
      rect.current = getRect(barRef.current);

      if (props.direction === 'down') {
        offsetRef.current = rect.current.bottom;
      } else {
        offsetRef.current = window.innerHeight - rect.current.top;
      }
    }
  };

  var closeAll = function closeAll() {
    childrenRefs.forEach(function (item) {
      item.toggle(false);
    });
    updateShowPopupIndex(null);
  };

  useImperativeHandle(ref, function () {
    return {
      close: closeAll
    };
  });

  var onClickAway = function onClickAway() {
    if (props.closeOnClickOutside) {
      closeAll();
    }
  };

  useEffect(function () {
    if (barRef.current) {
      updateOffset();
    }
  }, [barRef.current]);

  var onScroll = function onScroll() {
    if (opened) {
      updateOffset();
    }
  };

  var showItem = function showItem(index) {
    var item = childrenRefs[index];
    updateOffset();
    updateShowPopupIndex(index);
    item.toggle(true);
  };

  var onToggleItem = function onToggleItem(active) {
    childrenRefs.forEach(function (child, index) {
      if (index === active) {
        // 且连续点击同一个dropdown-menu__bar时，显示or隐藏
        if (active === showPopupIndexRef.current) {
          updateShowPopupIndex(null);
          child.toggle();
          return;
        }

        showItem(active);
      } else {
        child.toggle(false, {
          immediate: false
        });
      }
    });
  };

  var renderTitle = function renderTitle(item, index) {
    var showPopup = showPopupIndex === index;
    var disabled = item.disabled,
        titleClass = item.titleClass;
    return /*#__PURE__*/React.createElement("div", {
      key: index,
      role: "button",
      tabIndex: disabled ? -1 : 0,
      className: classnames(bem('item', {
        disabled: disabled
      }), _defineProperty({}, HAPTICS_FEEDBACK, !disabled)),
      onClick: function onClick() {
        if (!disabled) {
          onToggleItem(index);
        }
      }
    }, /*#__PURE__*/React.createElement("span", {
      className: classnames(bem('title', {
        down: showPopup === (props.direction === 'down'),
        active: showPopup
      }), titleClass),
      style: {
        color: showPopup ? props.activeColor : ''
      }
    }, /*#__PURE__*/React.createElement("div", {
      className: "rc-ellipsis"
    }, item.renderTitle())));
  };

  useClickAway(root, onClickAway);
  useEventListener('scroll', onScroll, {
    target: scrollParent
  });

  var handleChange = function handleChange(value) {
    var _props$onChange;

    if (titleFlag) {
      setTitleFlag(false);
    } else {
      setTitleFlag(true);
    }

    updateShowPopupIndex(null);
    (_props$onChange = props.onChange) === null || _props$onChange === void 0 ? void 0 : _props$onChange.call(props, value);
  };

  return /*#__PURE__*/React.createElement(DropdownMenuContext.Provider, {
    value: {
      props: props,
      offset: offsetRef.current,
      onChange: handleChange,
      close: closeAll
    }
  }, /*#__PURE__*/React.createElement("div", {
    ref: root,
    className: classnames(bem(), props.className),
    style: _objectSpread({}, props.style)
  }, /*#__PURE__*/React.createElement("div", {
    ref: barRef,
    style: barStyle(),
    className: classnames(bem('bar', {
      opened: opened
    }))
  }, childrenRefs.map(renderTitle)), React.Children.toArray(props.children).filter(Boolean).map(function (child, index) {
    return /*#__PURE__*/React.cloneElement(child, {
      ref: setChildrenRefs(index)
    });
  })));
});
DropdownMenu.defaultProps = {
  duration: 200,
  overlay: true,
  direction: 'down',
  zIndex: 2000,
  closeOnClickOutside: true,
  closeOnClickOverlay: true,
  shouldRenderTitle: false
};
DropdownMenu.displayName = 'DropdownMenu';
export default DropdownMenu;