import { useEffect } from 'react';
import { useTouch } from './use-touch';
import { getScrollParent } from './use-scroll-parent';
import { supportsPassive } from './use-support-passive';
var totalLockCount = 0;
var BODY_LOCK_CLASS = 'rc-overflow-hidden'; // eslint-disable-next-line @typescript-eslint/explicit-module-boundary-types

export function useLockScroll(rootRef, shouldLock) {
  var touch = useTouch();

  var onTouchMove = function onTouchMove(event) {
    touch.move(event);
    var direction = touch.deltaY.current > 0 ? '10' : '01';
    var el = getScrollParent(event.target, rootRef.current);
    if (!el) return;
    var scrollHeight = el.scrollHeight,
        offsetHeight = el.offsetHeight,
        scrollTop = el.scrollTop;
    var status = '11';

    if (scrollTop === 0) {
      status = offsetHeight >= scrollHeight ? '00' : '01';
    } else if (scrollTop + offsetHeight >= scrollHeight) {
      status = '10';
    }

    if (status !== '11' && touch.isVertical() && // eslint-disable-next-line no-bitwise
    !(parseInt(status, 2) & parseInt(direction, 2))) {
      if (event.cancelable) {
        event.preventDefault();
      }
    }
  };

  var lock = function lock() {
    document.addEventListener('touchstart', touch.start);
    document.addEventListener('touchmove', onTouchMove, supportsPassive ? {
      passive: false
    } : false);

    if (!totalLockCount) {
      document.body.classList.add(BODY_LOCK_CLASS);
    } // eslint-disable-next-line no-plusplus


    totalLockCount++;
  };

  var unlock = function unlock() {
    if (totalLockCount) {
      document.removeEventListener('touchstart', touch.start);
      document.removeEventListener('touchmove', onTouchMove); // eslint-disable-next-line no-plusplus

      totalLockCount--;

      if (!totalLockCount) {
        document.body.classList.remove(BODY_LOCK_CLASS);
      }
    }
  }; // eslint-disable-next-line consistent-return


  useEffect(function () {
    if (shouldLock) {
      lock();
      return function () {
        unlock();
      };
    }
  }, [shouldLock]);
}