import _defineProperty from "@babel/runtime/helpers/esm/defineProperty";
import _objectWithoutProperties from "@babel/runtime/helpers/esm/objectWithoutProperties";
import _slicedToArray from "@babel/runtime/helpers/esm/slicedToArray";
var _excluded = ["className", "type", "style", "image", "errorImage", "loading", "eventOptions", "height", "width", "onLoaded"];

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }

import React, { useEffect, useContext, createElement } from 'react';
import classNames from 'classnames';
import { addCssClassName, DEFAULT_EVENTS, DEFAULT_URL, loadImage, setImage } from './utils';
import useEventVisible from './utils/useEventVisible';
import ConfigProviderContext from '../config-provider/ConfigProviderContext';

var LazyloadImage = function LazyloadImage(props) {
  var _useContext = useContext(ConfigProviderContext),
      prefixCls = _useContext.prefixCls,
      createNamespace = _useContext.createNamespace;

  var _createNamespace = createNamespace('lazyload-image', prefixCls),
      _createNamespace2 = _slicedToArray(_createNamespace, 1),
      bem = _createNamespace2[0];

  var className = props.className,
      type = props.type,
      incomingStyle = props.style,
      image = props.image,
      errorImage = props.errorImage,
      loading = props.loading,
      eventOptions = props.eventOptions,
      height = props.height,
      width = props.width,
      onLoaded = props.onLoaded,
      restProps = _objectWithoutProperties(props, _excluded);

  var _useEventVisible = useEventVisible(eventOptions),
      _useEventVisible2 = _slicedToArray(_useEventVisible, 2),
      containerRef = _useEventVisible2[0],
      visible = _useEventVisible2[1];

  useEffect(function () {
    if (visible && containerRef.current) {
      var target = containerRef.current;
      var url = target.dataset.src || '';
      loadImage(url).catch(function () {
        setImage(target, errorImage);
        addCssClassName(target, bem('error'));
      }).then(function (imagePath) {
        var _props$onLoaded;

        setImage(target, imagePath);
        addCssClassName(target, bem('loaded'));
        target.setAttribute('lazy', 'loaded');
        target.removeAttribute('data-src');
        (_props$onLoaded = props.onLoaded) === null || _props$onLoaded === void 0 ? void 0 : _props$onLoaded.call(props);
      });
    }
  }, [visible]);

  var style = _objectSpread(_objectSpread({}, incomingStyle), {}, {
    height: height,
    width: width
  });

  return /*#__PURE__*/React.createElement(React.Fragment, null, type === 'image' ? /*#__PURE__*/createElement('img', _objectSpread(_objectSpread({}, restProps), {}, {
    ref: containerRef,
    style: style,
    'data-src': image,
    src: loading || DEFAULT_URL,
    className: classNames(bem(), className)
  })) : /*#__PURE__*/createElement('div', _objectSpread(_objectSpread({}, restProps), {}, {
    ref: containerRef,
    style: style,
    'data-src': image,
    className: classNames(bem(), className)
  }), props.children));
};

LazyloadImage.defaultProps = {
  loading: null,
  eventOptions: {
    listenEvents: DEFAULT_EVENTS,
    scrollContainer: document.body,
    offset: 0,
    debounce: 300,
    throttle: 0
  },
  type: 'image',
  errorImage: DEFAULT_URL
};
export default LazyloadImage;