import _defineProperty from "@babel/runtime/helpers/esm/defineProperty";
import _slicedToArray from "@babel/runtime/helpers/esm/slicedToArray";
import React, { forwardRef, useContext, useEffect, useState, useRef, useImperativeHandle } from 'react';
import classNames from 'classnames';
import ConfigProviderContext from '../config-provider/ConfigProviderContext';
import { BORDER_LEFT, BORDER_SURROUND } from '../utils/constant';
import { addUnit } from '../utils';
import useUpdateEffect from '../hooks/use-update-effect';
var DEFAULT_CELL_LENGTH = 6;
var PasswordInput = /*#__PURE__*/forwardRef(function (props, ref) {
  var _useContext = useContext(ConfigProviderContext),
      prefixCls = _useContext.prefixCls,
      createNamespace = _useContext.createNamespace;

  var _createNamespace = createNamespace('password-input', prefixCls),
      _createNamespace2 = _slicedToArray(_createNamespace, 1),
      bem = _createNamespace2[0];

  var rootRef = useRef(null);
  var nativeInputRef = useRef(null);

  var _useState = useState(props.value),
      _useState2 = _slicedToArray(_useState, 2),
      inputValue = _useState2[0],
      setInputValue = _useState2[1];

  var length = props.length,
      value = props.value,
      focused = props.focused;
  var info = props.errorInfo || props.info;
  var cellLength = length > 0 && length < Infinity ? Math.floor(length) : DEFAULT_CELL_LENGTH;
  useUpdateEffect(function () {
    if (['', undefined].includes(value)) {
      setInputValue('');
    }

    setInputValue(value.slice(0, cellLength));
  }, [value, cellLength]);
  useEffect(function () {
    var _props$onChange;

    if (inputValue.length >= cellLength) {
      var _props$onFill;

      (_props$onFill = props.onFill) === null || _props$onFill === void 0 ? void 0 : _props$onFill.call(props, inputValue);
    }

    (_props$onChange = props.onChange) === null || _props$onChange === void 0 ? void 0 : _props$onChange.call(props, inputValue.slice(0, cellLength));
  }, [props.onChange, props.onFill, inputValue, cellLength]);

  var onFocus = function onFocus() {
    var _props$onFocus;

    if (!props.keyboard) {
      var _nativeInputRef$curre;

      (_nativeInputRef$curre = nativeInputRef.current) === null || _nativeInputRef$curre === void 0 ? void 0 : _nativeInputRef$curre.focus();
    }

    (_props$onFocus = props.onFocus) === null || _props$onFocus === void 0 ? void 0 : _props$onFocus.call(props);
  };

  var onBlur = function onBlur() {
    var _props$onBlur;

    (_props$onBlur = props.onBlur) === null || _props$onBlur === void 0 ? void 0 : _props$onBlur.call(props);
  };

  useImperativeHandle(ref, function () {
    return {
      focus: function focus() {
        return onFocus();
      },
      blur: function blur() {
        var _nativeInputRef$curre2;

        onBlur();
        (_nativeInputRef$curre2 = nativeInputRef.current) === null || _nativeInputRef$curre2 === void 0 ? void 0 : _nativeInputRef$curre2.blur();
      }
    };
  });

  var onTouchStart = function onTouchStart(event) {
    var _props$onFocus2;

    event.stopPropagation();
    (_props$onFocus2 = props.onFocus) === null || _props$onFocus2 === void 0 ? void 0 : _props$onFocus2.call(props);
  };

  var renderPoints = function renderPoints() {
    var Points = [];
    var mask = props.mask,
        gutter = props.gutter;

    for (var i = 0; i < length; i++) {
      var _char = inputValue[i];
      var showBorder = i !== 0 && !gutter;
      var showCursor = focused && i === inputValue.length;
      var style = void 0;

      if (i !== 0 && gutter) {
        style = {
          marginLeft: addUnit(gutter)
        };
      }

      Points.push( /*#__PURE__*/React.createElement("li", {
        key: i,
        className: classNames(_defineProperty({}, BORDER_LEFT, showBorder), bem('item', {
          focus: showCursor
        })),
        style: style
      }, mask ? /*#__PURE__*/React.createElement("i", {
        style: {
          visibility: _char ? 'visible' : 'hidden'
        }
      }) : _char, showCursor && /*#__PURE__*/React.createElement("div", {
        className: classNames(bem('cursor'))
      })));
    }

    return Points;
  };

  return /*#__PURE__*/React.createElement(React.Fragment, null, /*#__PURE__*/React.createElement("div", {
    ref: rootRef // eslint-disable-next-line jsx-a11y/no-noninteractive-tabindex
    ,
    tabIndex: 0,
    onFocus: onFocus,
    onBlur: onBlur,
    className: classNames(bem())
  }, /*#__PURE__*/React.createElement("ul", {
    className: classNames(bem('security'), _defineProperty({}, BORDER_SURROUND, !props.gutter)),
    onTouchStart: onTouchStart
  }, renderPoints()), info && /*#__PURE__*/React.createElement("div", {
    className: classNames(bem(props.errorInfo ? 'error-info' : 'info'))
  }, info), /*#__PURE__*/React.createElement("input", {
    ref: nativeInputRef,
    className: classNames(bem('native-input')),
    value: inputValue,
    type: "text",
    pattern: "[0-9]*",
    inputMode: "numeric",
    onChange: function onChange(e) {
      setInputValue(e.target.value.slice(0, props.length));
    }
  })), props.keyboard && /*#__PURE__*/React.cloneElement(props.keyboard, {
    visible: focused,
    onInput: function onInput(v) {
      if (inputValue.length < cellLength) {
        setInputValue((inputValue + v).slice(0, props.length));
      }
    },
    onDelete: function onDelete() {
      setInputValue(inputValue.slice(0, -1));
    },
    onClose: function onClose() {
      var _rootRef$current;

      (_rootRef$current = rootRef.current) === null || _rootRef$current === void 0 ? void 0 : _rootRef$current.blur();
    },
    onBlur: function onBlur() {
      var _rootRef$current2;

      (_rootRef$current2 = rootRef.current) === null || _rootRef$current2 === void 0 ? void 0 : _rootRef$current2.blur();
    }
  }));
});
PasswordInput.defaultProps = {
  value: '',
  info: null,
  errorInfo: null,
  length: DEFAULT_CELL_LENGTH,
  gutter: 0,
  mask: true,
  focused: false,
  onChange: function onChange() {}
};
export default PasswordInput;