import _extends from "@babel/runtime/helpers/esm/extends";
import _slicedToArray from "@babel/runtime/helpers/esm/slicedToArray";

/* eslint-disable @typescript-eslint/no-shadow */
import React, { forwardRef, useContext, useRef, useImperativeHandle } from 'react';
import classNames from 'classnames';
import { useSpring, animated } from '@react-spring/web';
import { useDrag } from '@use-gesture/react';
import useClickAway from '../hooks/use-click-away';
import useMountedRef from '../hooks/use-mounted-ref';
import ConfigProviderContext from '../config-provider/ConfigProviderContext';
import { callInterceptor } from '../utils/interceptor';
import { nearest } from '../utils/nearest';
import { isDef } from '../utils';
var SwipeCell = /*#__PURE__*/forwardRef(function (props, ref) {
  var _useContext = useContext(ConfigProviderContext),
      prefixCls = _useContext.prefixCls,
      createNamespace = _useContext.createNamespace;

  var _createNamespace = createNamespace('swipe-cell', prefixCls),
      _createNamespace2 = _slicedToArray(_createNamespace, 1),
      bem = _createNamespace2[0];

  var rootRef = useRef(null);
  var leftRef = useRef();
  var rightRef = useRef();
  var opened = useRef(false);
  var lockClick = useRef(false);
  var draggingRef = useRef(false);
  var mountedRef = useMountedRef();

  var getWidth = function getWidth(elementRef) {
    return elementRef.current ? elementRef.current.offsetWidth : 0;
  };

  function getLeftWidth() {
    return isDef(props.leftWidth) ? +props.leftWidth : getWidth(leftRef);
  }

  function getRightWidth() {
    return isDef(props.rightWidth) ? +props.rightWidth : getWidth(rightRef);
  }

  var _useSpring = useSpring(function () {
    return {
      x: 0,
      config: {
        tension: 200,
        friction: 30
      }
    };
  }, []),
      _useSpring2 = _slicedToArray(_useSpring, 2),
      x = _useSpring2[0].x,
      api = _useSpring2[1];

  var bind = useDrag(function (state) {
    if (!props.disabled) {
      lockClick.current = true;
      draggingRef.current = true;

      var _state$offset = _slicedToArray(state.offset, 1),
          offsetX = _state$offset[0];

      if (state.last) {
        var leftWidth = getLeftWidth();
        var rightWidth = getRightWidth();
        var position = offsetX + state.velocity[0] * state.direction[0] * 50;

        if (offsetX > 0) {
          position = Math.max(0, position);
        } else {
          position = Math.min(0, position);
        }

        var nearestPosition = nearest([-rightWidth, 0, leftWidth], position);
        api.start({
          x: nearestPosition
        });
        opened.current = nearestPosition !== 0;
        window.setTimeout(function () {
          if (mountedRef.current) {
            draggingRef.current = false;
            lockClick.current = false;
          }
        });
      } else {
        api.start({
          x: offsetX,
          immediate: true
        });
      }
    }
  }, {
    from: function from() {
      return [x.get(), 0];
    },
    bounds: function bounds() {
      var leftWidth = getLeftWidth();
      var rightWidth = getRightWidth();
      return {
        left: -rightWidth,
        right: leftWidth
      };
    },
    axis: 'x',
    preventScroll: true,
    pointer: {
      touch: true
    }
  });

  var _open = function open(side) {
    var _props$onOpen;

    opened.current = true;
    var offset = side === 'left' ? getLeftWidth() : -getRightWidth();
    api.start({
      x: offset
    });
    (_props$onOpen = props.onOpen) === null || _props$onOpen === void 0 ? void 0 : _props$onOpen.call(props, {
      position: side,
      name: props.name
    });
  };

  var _close = function close(position) {
    api.start({
      x: 0
    });

    if (opened.current) {
      var _props$onClose;

      opened.current = false;
      (_props$onClose = props.onClose) === null || _props$onClose === void 0 ? void 0 : _props$onClose.call(props, {
        position: position,
        name: props.name
      });
    }
  };

  var onClick = function onClick() {
    var _props$onClick;

    var position = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 'outside';
    (_props$onClick = props.onClick) === null || _props$onClick === void 0 ? void 0 : _props$onClick.call(props, position);

    if (opened.current && !lockClick.current) {
      callInterceptor({
        interceptor: props.beforeClose,
        args: [{
          name: props.name,
          position: position,
          instance: {
            close: function close() {
              return _close(position);
            }
          }
        }],
        done: function done() {
          return _close(position);
        }
      });
    }
  };

  var getClickHandler = function getClickHandler(position, stop) {
    return function (event) {
      if (stop) {
        event.preventDefault();
        event.stopPropagation();
      }

      onClick(position);
    };
  };

  var renderSideContent = function renderSideContent(side, sideRef) {
    var contentSlot = props[side];
    return contentSlot ? /*#__PURE__*/React.createElement("div", {
      ref: sideRef,
      className: classNames(bem(side)),
      onClick: getClickHandler(side, true)
    }, contentSlot) : null;
  };

  useImperativeHandle(ref, function () {
    return {
      open: function open(side) {
        return _open(side);
      },
      close: _close
    };
  });
  useClickAway(rootRef, function () {
    return onClick('outside');
  }, 'touchstart', props.hideOnClickOutside);
  return /*#__PURE__*/React.createElement("div", _extends({
    ref: rootRef
  }, bind(), {
    className: classNames(bem()),
    onClickCapture: function onClickCapture(e) {
      if (draggingRef.current) {
        e.stopPropagation();
        e.preventDefault();
      }
    },
    onClick: getClickHandler('cell')
  }), /*#__PURE__*/React.createElement(animated.div, {
    className: classNames(bem('wrapper')),
    style: {
      x: x
    }
  }, renderSideContent('left', leftRef), /*#__PURE__*/React.createElement("div", {
    className: classNames(bem('content')),
    onClickCapture: function onClickCapture(e) {
      if (x.goal !== 0) {
        e.preventDefault();
        e.stopPropagation();
        api.start({
          x: 0
        });
      }
    }
  }, /*#__PURE__*/React.createElement(animated.div, {
    style: {
      pointerEvents: x.to(function (v) {
        return v !== 0 && x.goal !== 0 ? 'none' : 'unset';
      })
    }
  }, props.children)), renderSideContent('right', rightRef)));
});
SwipeCell.defaultProps = {
  disabled: false,
  hideOnClickOutside: true
};
export default SwipeCell;