"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.calcDateNum = calcDateNum;
exports.cloneDates = exports.cloneDate = void 0;
exports.compareDay = compareDay;
exports.compareMonth = compareMonth;
exports.formatMonthTitle = void 0;
exports.getDayByOffset = getDayByOffset;
exports.t = exports.getToday = exports.getPrevDay = exports.getNextDay = exports.getMonthEndDay = void 0;

/* eslint-disable no-nested-ternary */
function compareMonth(date1, date2) {
  var year1 = date1.getFullYear();
  var year2 = date2.getFullYear();

  if (year1 === year2) {
    var month1 = date1.getMonth();
    var month2 = date2.getMonth();
    return month1 === month2 ? 0 : month1 > month2 ? 1 : -1;
  }

  return year1 > year2 ? 1 : -1;
}

function compareDay(day1, day2) {
  var compareMonthResult = compareMonth(day1, day2);

  if (compareMonthResult === 0) {
    var date1 = day1.getDate();
    var date2 = day2.getDate();
    return date1 === date2 ? 0 : date1 > date2 ? 1 : -1;
  }

  return compareMonthResult;
}

var cloneDate = function cloneDate(date) {
  return new Date(date);
};

exports.cloneDate = cloneDate;

var cloneDates = function cloneDates(dates) {
  return Array.isArray(dates) ? dates.map(cloneDate) : cloneDate(dates);
};

exports.cloneDates = cloneDates;

function getDayByOffset(date, offset) {
  var cloned = cloneDate(date);
  cloned.setDate(cloned.getDate() + offset);
  return cloned;
}

var getPrevDay = function getPrevDay(date) {
  return getDayByOffset(date, -1);
};

exports.getPrevDay = getPrevDay;

var getNextDay = function getNextDay(date) {
  return getDayByOffset(date, 1);
};

exports.getNextDay = getNextDay;

var getToday = function getToday() {
  var today = new Date();
  today.setHours(0, 0, 0, 0);
  return today;
};

exports.getToday = getToday;

function calcDateNum(date) {
  var day1 = date[0].getTime();
  var day2 = date[1].getTime();
  return (day2 - day1) / (1000 * 60 * 60 * 24) + 1;
}

var getMonthEndDay = function getMonthEndDay(year, month) {
  return 32 - new Date(year, month - 1, 32).getDate();
};

exports.getMonthEndDay = getMonthEndDay;
var calendarLang = {
  end: '结束',
  start: '开始',
  title: '日期选择',
  confirm: '确定',
  startEnd: '开始/结束',
  weekdays: ['日', '一', '二', '三', '四', '五', '六'],
  monthTitle: function monthTitle(year, month) {
    return "".concat(year, "\u5E74").concat(month, "\u6708");
  },
  rangePrompt: function rangePrompt(maxRange) {
    return "\u6700\u591A\u9009\u62E9 ".concat(maxRange, " \u5929");
  }
};

var t = function t(key) {
  for (var _len = arguments.length, args = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    args[_key - 1] = arguments[_key];
  }

  if (args.length) {
    return calendarLang[key].apply(calendarLang, args);
  }

  return calendarLang[key];
};

exports.t = t;

var formatMonthTitle = function formatMonthTitle(date) {
  return t('monthTitle', date.getFullYear(), date.getMonth() + 1);
};

exports.formatMonthTitle = formatMonthTitle;