"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.DEFAULT_URL = exports.DEFAULT_EVENTS = void 0;
exports.addCssClassName = addCssClassName;
exports.call = void 0;
exports.debounce = debounce;
exports.hasIntersectionObserver = void 0;
exports.loadImage = loadImage;
exports.modeType = void 0;
exports.setImage = setImage;
exports.throttle = throttle;

/* eslint-disable prefer-rest-params */
var inBrowser = typeof window !== 'undefined';
var DEFAULT_URL = 'data:image/gif;base64,R0lGODlhAQABAIAAAAAAAP///yH5BAEAAAAALAAAAAABAAEAAAIBRAA7'; // 需要监听的事件

exports.DEFAULT_URL = DEFAULT_URL;
var DEFAULT_EVENTS = ['scroll', 'wheel', 'resize', 'mousewheel', 'animationend', 'transitionend', 'touchmove'];
exports.DEFAULT_EVENTS = DEFAULT_EVENTS;
var hasIntersectionObserver = inBrowser && 'IntersectionObserver' in window && 'IntersectionObserverEntry' in window && 'intersectionRatio' in window.IntersectionObserverEntry.prototype;
exports.hasIntersectionObserver = hasIntersectionObserver;
var modeType = {
  event: 'event',
  observer: 'observer'
};
exports.modeType = modeType;

var call = function call(fn) {
  return fn && fn();
};

exports.call = call;

function setImage(element, imagePath) {
  if (element instanceof HTMLIFrameElement || element instanceof HTMLImageElement || element instanceof HTMLMediaElement) {
    element.src = imagePath;
  } else {
    element.style.backgroundImage = "url('".concat(imagePath, "')");
  }
}

function loadImage(imagePath) {
  return new Promise(function (resolve, reject) {
    var img = new Image();
    img.src = imagePath;

    img.onload = function () {
      return resolve(imagePath);
    };

    img.onerror = function (err) {
      return reject(err);
    };
  });
}

function addCssClassName(element, cssClassName) {
  if (!element.className.includes(cssClassName)) {
    element.className += " ".concat(cssClassName);
  }
}

function throttle(fn) {
  var _arguments = arguments,
      _this = this;

  var threshold = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 250;
  var scope = arguments.length > 2 ? arguments[2] : undefined;
  var last;
  var deferTimer;
  return function () {
    var context = scope || _this;
    var now = +new Date();
    var args = _arguments;

    if (last && now < last + threshold) {
      // hold on to it
      clearTimeout(deferTimer);
      deferTimer = setTimeout(function () {
        last = now;
        fn.apply(context, args);
      }, threshold);
    } else {
      last = now;
      fn.apply(context, args);
    }
  };
}

function debounce(func, wait, immediate) {
  var timeout;
  var args;
  var context;
  var timestamp;
  var result;

  var later = function later() {
    var last = +new Date() - timestamp;

    if (last < wait && last >= 0) {
      timeout = setTimeout(later, wait - last);
    } else {
      timeout = null;

      if (!immediate) {
        result = func.apply(context, args);

        if (!timeout) {
          context = null;
          args = null;
        }
      }
    }
  };

  return function debounced() {
    context = this;
    args = arguments;
    timestamp = +new Date();
    var callNow = immediate && !timeout;

    if (!timeout) {
      timeout = setTimeout(later, wait);
    }

    if (callNow) {
      result = func.apply(context, args);
      context = null;
      args = null;
    }

    return result;
  };
}